package com.ccalvouoc.dao;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.criterion.Restrictions;

import com.ccalvouoc.orm.Contractor;
import com.ccalvouoc.orm.ControlVisit;
import com.ccalvouoc.orm.ControlVisitId;
import com.ccalvouoc.orm.Coordinator;
import com.ccalvouoc.orm.News;
import com.ccalvouoc.orm.Project;
import com.ccalvouoc.orm.ProjectHasNews;
import com.ccalvouoc.orm.ProjectHasNewsId;
import com.ccalvouoc.orm.Status;
import com.ccalvouoc.orm.Technician;
import com.ccalvouoc.orm.User;
import com.ccalvouoc.orm.VisitEpcs;
import com.ccalvouoc.orm.VisitEpis;
import com.ccalvouoc.orm.VisitSignage;
import com.ccalvouoc.utils.FileUtils;

public class ControlVisitDao {
	private static final Logger logger = Logger.getLogger(ControlVisitDao.class.getName());
	Session session;
	
	
	@SuppressWarnings("unchecked")
	public ControlVisit getControlVisit(String projectCode, Date date){
		ControlVisit controlVisit = null; 
		
		
		try{
			
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			
			
			Criteria criteria = session.createCriteria(ControlVisit.class);
			
			criteria.add(Restrictions.eq("id.projectId",projectCode));
			//System.out.println("FEcha:>>>>>"+date);
			criteria.add(Restrictions.ge("id.date",date ));
			List<ControlVisit> results = criteria.list();
			if(results==null || results.isEmpty()){
				return null;
			}
			controlVisit= results.get(0);
			
			
			
		}catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if(session!=null && session.isOpen())
				session.close();
		}
		
		return controlVisit;
		
	}
	
 	@SuppressWarnings("unchecked")
public boolean  removeControlVisit(ControlVisit controlVisit) {

		try {
			
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();			
			String pathFiles =  getFilePath(controlVisit);
			ControlVisit controlVisitToBeDeleted =   (ControlVisit)session.merge(controlVisit); 			
			session.beginTransaction();
			session.delete(controlVisitToBeDeleted);		
			FileUtils.deleteFolderWithFiles(pathFiles);			
			session.getTransaction().commit();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}
		
		return true;
	
	}
	
	@SuppressWarnings("unchecked")
	public ControlVisit saveControlVisit(ControlVisit controlVisit) {

		
		try {
	
			SessionFactory sessionFactory = createSessionFactory();
			
			 Session session = sessionFactory.openSession();
			 
			session.beginTransaction();
			
			
			
			session.merge(controlVisit);
			session.merge(controlVisit.getVisitEpis());
			
			session.merge(controlVisit.getVisitEpcs());
			session.merge(controlVisit.getVisitSignage());
			
			
			
			session.getTransaction().commit();
			
			

		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			
			session.getTransaction().rollback();
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
			
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}

		return getControlVisit(controlVisit.getId().getProjectId(), controlVisit.getId().getDate());
	}
	
	
	@SuppressWarnings("unchecked")
	public boolean createNewControlVisit(String code, Date date) {

		try {
		
			
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			session.beginTransaction();
			
			//creo la clave de la visita de control
			ControlVisitId controlVisitId = new ControlVisitId(code, date);
			
			
			//Creo la visita y las epcs, epis, signage
			ControlVisit controlVisit = new ControlVisit();
			VisitEpcs visitEpcs= new VisitEpcs();
			VisitEpis visitEpis = new VisitEpis();
			VisitSignage visitSignage = new VisitSignage();
			
			
			//asigno la id
			controlVisit.setId(controlVisitId);
			visitEpcs.setId(controlVisitId);
			visitEpis.setId(controlVisitId);
			visitSignage.setId(controlVisitId);
			
			//inicializo valores
			controlVisit.setReport("");
			visitEpcs.setComments("");
			visitEpis.setComments("");
			visitSignage.setComments("");
			
			Status status = new Status(2);
			visitEpcs.setStatus(status);
			visitEpis.setStatus(status);
			visitSignage.setStatus(status);
			
			visitEpcs.setControlVisit(controlVisit);
			visitEpis.setControlVisit(controlVisit);
			visitSignage.setControlVisit(controlVisit);
			
			
			//guardo la visita
			session.save(controlVisit);
			session.save(visitEpcs);
			session.save(visitEpis);
			session.save(visitSignage);


			session.getTransaction().commit();

		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
			
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}

		return true;
	}
	
	private String getFilePath(ControlVisit controlVisit) {
		String filePath = "";
    	Date date = controlVisit.getId().getDate();

    	SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
    	String formattedDate = formatter.format(date);
    	
		filePath = "PSSfiles" +  File.separator + controlVisit.getProject().getId() +  File.separator + formattedDate;
		
        //FilesUtil.saveFile(getFile(), getFileFileName(), context.getRealPath("") + File.separator + filesPath);
    	String directorioRaiz = System.getenv("OPENSHIFT_DATA_DIR");
    	if(directorioRaiz !=null && !directorioRaiz.equals("")) {
    		filePath = directorioRaiz + File.separator + filePath;        		
    	} else {
    		filePath = "C:" + File.separator + filePath;  
    	}
    	return filePath;		
	}
	
	public static SessionFactory createSessionFactory() {
		SessionFactory sessionFactory = new AnnotationConfiguration().configure().buildSessionFactory();
	    return sessionFactory;
	}

}
