package com.ccalvouoc.dao;


import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import com.ccalvouoc.orm.Contractor;
import com.ccalvouoc.orm.ControlVisit;
import com.ccalvouoc.orm.Coordinator;
import com.ccalvouoc.orm.Documents;
import com.ccalvouoc.orm.News;
import com.ccalvouoc.orm.Project;
import com.ccalvouoc.orm.ProjectHasNews;
import com.ccalvouoc.orm.ProjectHasNewsId;
import com.ccalvouoc.orm.SiteManager;
import com.ccalvouoc.orm.Technician;

public class ProjectDao {
	private static final Logger logger = Logger.getLogger(ProjectDao.class
			.getName());
	Session session;

	@SuppressWarnings("unchecked")
	public List<Project> listProjects() {
		List<Project> listProjects = null;
		try {
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			
			Criteria criteria = session.createCriteria(Project.class);
			
			criteria.addOrder(Order.desc("created"));
			
			listProjects = criteria.list();
			
		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}
		return listProjects;
	}

	@SuppressWarnings("unchecked")
	public Project getProject(String code) {
		Project project = null;
		
		try {
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();

			Criteria criteria = session.createCriteria(Project.class);
			
			criteria.add(Restrictions.eq("id", code));
			project = (Project) criteria.list().get(0);

			/*
			 * String hql = "FROM Project"; Query query =
			 * session.createQuery(hql); listaNames = query.list();
			 * session.getTransaction().commit();
			 */
		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}
		return project;
	}

	public String createNewProject(String title, int technicianId,
			int coordinatorId, int contractorId) {

		// logger.warn("** EJECUTANDO addActividad() **");
		String projectCode = getNewProjectCode();
		try {
		

			Date d = new Date();

			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			session.beginTransaction();

			Project project = new Project();
			project.setId(projectCode);
			project.setTitle(title);
			project.setCreated(d);

			
			//recupero el tecnico, el contratista y el coordinador de la obra
			TechnicianDao technicianDao = new TechnicianDao();
			ContractorDao contractorDao = new ContractorDao();
			CoordinatorDao coordinatorDao = new CoordinatorDao();

			Technician technician = technicianDao.getTechnician(technicianId);
			Contractor contractor = contractorDao.getContractor(contractorId);
			Coordinator coordinator = coordinatorDao
					.getCoordinator(coordinatorId);
			
			
			// asigno el tecnico, el contratista y el coordinador a la obra
			
			project.setTechnician(technician);
			project.setContractor(contractor);
			project.setCoordinator(coordinator);

			
			// añado a la obra la noticia " Nueva obra"
			NewsDao newsDao = new NewsDao();
			News newsNewProject = newsDao.getNews(1);
			ProjectHasNews projectHasNews = new ProjectHasNews();
			ProjectHasNewsId  projectHasNewsId= new ProjectHasNewsId(project.getId(),newsNewProject.getId());
			projectHasNews.setId(projectHasNewsId);
			projectHasNews.setNews(newsNewProject);
			projectHasNews.setDateNews(new Date());

			Set<ProjectHasNews> listProjectHasNews = new HashSet<ProjectHasNews>();
			listProjectHasNews.add(projectHasNews);
			
			project.setProjectHasNewses(listProjectHasNews);			
			session.save(project);
			session.save(projectHasNews);
			
			
			//Se graba el objetos documents 		
			Documents documents = new Documents(project);
			session.save(documents);
		
			 
			session.getTransaction().commit();

		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
			
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}

		return projectCode;
	}
	
public Project saveProject(Project project) {

		
		try {
			
			SessionFactory sessionFactory = createSessionFactory();
			
			 session = sessionFactory.openSession();
			 
			session.beginTransaction();
			
			
			
			//Si el SiteManager todavía no existe en tablas, lo grabamos y actualizamos el id
			if(project.getSiteManager().getId()==0  ) {
				SiteManagerDao siteManagerDao = new SiteManagerDao();
				int idSiteManager = siteManagerDao.addSiteManager(project.getSiteManager());
				project.getSiteManager().setId(idSiteManager);	
				
			} 
			SiteManager updatedSiteManager = (SiteManager)session.merge(project.getSiteManager());			
			project.setSiteManager(updatedSiteManager);
			session.merge(project.getDocuments());			
			session.merge(project);			
			
			
			
			session.getTransaction().commit();
			
			

		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			
			session.getTransaction().rollback();
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
			
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}

		return getProject(project.getId());
	}

public Project closeProject(String code, Date date, String closeComments) {

	
	try {
		Project project = getProject(code);
		
		
		SessionFactory sessionFactory = createSessionFactory();
		
		 Session session = sessionFactory.openSession();
		 
		session.beginTransaction();
		 
		
		
		// set date y comments
		project.setCloseComments(closeComments);
		project.setDataClose(date);
		session.merge(project);
		

		session.getTransaction().commit();
		
		addNewsClosedProject(project);
		

	} catch (HibernateException e) {
		logger.error(e.getMessage(), e);
		
		session.getTransaction().rollback();
		
	} catch (Exception e) {
		logger.error(e.getMessage(), e);
		session.getTransaction().rollback();
		
	} finally {
		if (session != null && session.isOpen())
			session.close();
	}

	return getProject(code);
}
	
	public Project addNewsControlVisit(Project project){
		NewsDao newsDao = new NewsDao();
		News newsControlVisit= newsDao.getNews(3);

		ProjectHasNews projectHasNews = new ProjectHasNews();
		ProjectHasNewsId  projectHasNewsId= new ProjectHasNewsId(project.getId(),newsControlVisit.getId());
		projectHasNews.setId(projectHasNewsId);
		projectHasNews.setNews(newsControlVisit);
		projectHasNews.setDateNews(new Date());

		
		project.getProjectHasNewses().add(projectHasNews);
		
		
		
		try {
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			session.beginTransaction();
			session.save(projectHasNews);
			
			session.getTransaction().commit();
		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}
		
		
		return project;
		
	}
	public Project addNewsClosedProject(Project project){
		
		
		NewsDao newsDao = new NewsDao();
		News newsControlVisit= newsDao.getNews(4);
		
		
		
		ProjectHasNews projectHasNews = new ProjectHasNews();
		ProjectHasNewsId  projectHasNewsId= new ProjectHasNewsId(project.getId(),newsControlVisit.getId());
		projectHasNews.setId(projectHasNewsId);
		projectHasNews.setNews(newsControlVisit);
		projectHasNews.setDateNews(new Date());

		
		project.getProjectHasNewses().add(projectHasNews);
		
		
		try {
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();
			session.beginTransaction();
			session.save(projectHasNews);
			
			session.getTransaction().commit();
		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}
		
		
		return project;
		
	}

	
	/*Generamos el nuevo codigo con formato OByy-999
	 * yy: año
	 * 999: numero secuencial
	 * */
	private String getNewProjectCode() {
		
		int numProject = 1;
		String newCodeProject = null;
		String newCodeProjectPrefix = "OB" + getYearFormatedTwoDigits() + "-";
		

		try {
			SessionFactory sessionFactory = createSessionFactory();
			Session session = sessionFactory.openSession();

			Criteria criteria = session.createCriteria(Project.class);
			
			criteria.add(Restrictions.like("id", newCodeProjectPrefix + "%"));
			criteria.addOrder(Order.desc("id"));
			if (criteria.list().size() != 0) {
				Project lastProject = (Project) criteria.list().get(0);
			
				numProject = Integer.parseInt(lastProject.getId().substring(5,
						8)) + 1;
				

			}
			newCodeProject = newCodeProjectPrefix
					+ String.format("%03d", numProject);

			
		} catch (HibernateException e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			session.getTransaction().rollback();
		} finally {
			if (session != null && session.isOpen())
				session.close();
		}

		return newCodeProject;

	}



	private String getYearFormatedTwoDigits() {
		//recupera el año actual en formato yy
		SimpleDateFormat sdf = new SimpleDateFormat("yy"); // Just the year,
															// with 2 digits
		String formattedDate = sdf.format(Calendar.getInstance().getTime());
		return formattedDate;
	}
	
	

	public static SessionFactory createSessionFactory() {
		SessionFactory sessionFactory = new AnnotationConfiguration()
				.configure().buildSessionFactory();
		return sessionFactory;
	}

}