package com.ccalvouoc.utils;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.ImageIO;

import org.apache.log4j.Logger;

public class FileUtils {
	
	
	private static final Logger logger = Logger.getLogger(FileUtils.class.getName());
	
	private static final int MAX_IMAGE_SIZE = 200;
	
    public static void saveFileOnDisk(File file, String fileName, String fileDirectory) throws IOException{
        FileInputStream in = null;
        FileOutputStream out = null;
         
        File dir = new File (fileDirectory);
        if ( !dir.exists() )
           dir.mkdirs();
         
        String targetPath =  dir.getPath() + File.separator + fileName;
        File destinationFile = new File ( targetPath);
        try {
            in = new FileInputStream( file );
            out = new FileOutputStream( destinationFile );
            int c;
 
            while ((c = in.read()) != -1) {
                out.write(c);
            }
 
        }finally {
            if (in != null) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
        }         
    }
    
        
    public static void saveThumbnail(File imagefile, String fileName, String fileDirectory) throws Exception{
    	
    	try {
    		BufferedImage originalImage = ImageIO.read(imagefile);
    		BufferedImage resizeImageJpg = resizeImage(originalImage);
    	    File dir = new File (fileDirectory);
	        if ( !dir.exists() )
	           dir.mkdirs();
    		String targetPath =  dir.getPath() + File.separator + fileName;
    		ImageIO.write(resizeImageJpg, "jpg", new File(targetPath)); 
    	}catch(Exception ex) {
    		logger.error(ex.getMessage(), ex);
    		throw ex;
    	}	
    	
    }
    
    
    
    public static InputStream getFile(String filePath, String fileName) throws Exception{
    	FileInputStream fis = null;    	
    	try{
            File fileToDownload = new File(filePath + File.separator + fileName);
            fis = new FileInputStream(fileToDownload);
    		
    	} catch(Exception ex) {
    		logger.error(ex.getMessage(), ex);
    		throw ex;
    	}
    	return fis;    	
    }
    
    
    private static BufferedImage resizeImage(BufferedImage originalImage){
    	
    	double imageHeight = originalImage.getHeight();
    	double imageWidth = originalImage.getWidth();
    	
    	double proportion = MAX_IMAGE_SIZE/imageHeight < MAX_IMAGE_SIZE/imageWidth ? MAX_IMAGE_SIZE/imageHeight : MAX_IMAGE_SIZE/imageWidth;  	
    	
    	int newWidth = (int)(imageWidth*proportion);
    	int newHeight = (int)(imageHeight*proportion);
    	
    	BufferedImage resizedImage = new BufferedImage(newWidth, newHeight, originalImage.getType());
    	Graphics2D g = resizedImage.createGraphics();
    	g.drawImage(originalImage, 0, 0, newWidth, newHeight, null);
    	g.dispose();     
    	return resizedImage;
   }
     

    public static void deleteFolderWithFiles(String pathToFolder) throws Exception {
    	File folder = new File(pathToFolder);
    	String[]entries = folder.list();
    	for(String s: entries){
    	    File currentFile = new File(folder.getPath(),s);
    	    currentFile.delete();
    	}
    	folder.delete();
    }
	

}
